@@ -13,28 +13,27 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import with_statement
-
 import os
+import sys
 import random
 
-from nose.plugins.skip import SkipTest
+import pytest
 
 from mapproxy.cache.riak import RiakCache
-from mapproxy.grid import tile_grid
 from mapproxy.compat.modules import urlparse
+from mapproxy.grid import tile_grid
 from mapproxy.test.image import create_tmp_image_buf
 from mapproxy.test.unit.test_cache_tile import TileCacheTestBase
 
+
 tile_image = create_tmp_image_buf((256, 256), color='blue')
 tile_image2 = create_tmp_image_buf((256, 256), color='red')
 
+
+@pytest.mark.skipif(sys.version_info > (3, 7), reason="riak is not compatible with this Python version")
 class RiakCacheTestBase(TileCacheTestBase):
     always_loads_metadata = True
     def setup(self):
-        if not os.environ.get(self.riak_url_env):
-            raise SkipTest()
-
         url = os.environ[self.riak_url_env]
         urlparts = urlparse.urlparse(url)
         protocol = urlparts.scheme.lower()
@@ -49,8 +48,7 @@ class RiakCacheTestBase(TileCacheTestBase):
 
         TileCacheTestBase.setup(self)
 
-        self.cache = RiakCache([node], protocol, db_name, tile_grid=tile_grid(3857, name='global-webmarcator'),
-                lock_dir=self.cache_dir)
+        self.cache = RiakCache([node], protocol, db_name, tile_grid=tile_grid(3857, name='global-webmarcator'))
 
     def teardown(self):
         import riak
@@ -65,8 +63,14 @@ class RiakCacheTestBase(TileCacheTestBase):
         assert self.cache.remove_tile(tile)
         assert self.cache.remove_tile(tile)
 
+
+@pytest.mark.skipif(not os.environ.get('MAPPROXY_TEST_RIAK_HTTP'),
+                    reason="MAPPROXY_TEST_RIAK_HTTP not set")
 class TestRiakCacheHTTP(RiakCacheTestBase):
     riak_url_env = 'MAPPROXY_TEST_RIAK_HTTP'
 
+
+@pytest.mark.skipif(not os.environ.get('MAPPROXY_TEST_RIAK_PBC'),
+                    reason="MAPPROXY_TEST_RIAK_PBC not set")
 class TestRiakCachePBC(RiakCacheTestBase):
-    riak_url_env = 'MAPPROXY_TEST_RIAK_PBC'
\ No newline at end of file
+    riak_url_env = 'MAPPROXY_TEST_RIAK_PBC'
