@@ -16,6 +16,7 @@
 from __future__ import print_function, division
 
 import sys
+from collections import deque
 from contextlib import contextmanager
 import time
 try:
@@ -32,7 +33,7 @@ from mapproxy.util.lock import LockTimeout
 from mapproxy.seed.util import format_seed_task, timestamp
 from mapproxy.seed.cachelock import DummyCacheLocker, CacheLockedError
 
-from mapproxy.seed.util import (exp_backoff, ETA, limit_sub_bbox,
+from mapproxy.seed.util import (exp_backoff, limit_sub_bbox,
     status_symbol, BackoffError)
 
 import logging
@@ -54,40 +55,11 @@ else:
     queue_class = multiprocessing.Queue
 
 
-class TileProcessor(object):
-    def __init__(self, dry_run=False):
-        self._lastlog = time.time()
-        self.dry_run = dry_run
-
-    def log_progress(self, progress):
-        if (self._lastlog + .1) < time.time():
-            # log progress at most every 100ms
-            print('[%s] %6.2f%% %s \tETA: %s\r' % (
-                timestamp(), progress[1]*100, progress[0],
-                progress[2]
-            ), end=' ')
-            sys.stdout.flush()
-            self._lastlog = time.time()
-
-    def process(self, tiles, progress):
-        if not self.dry_run:
-            self.process_tiles(tiles)
-
-        self.log_progress(progress)
-
-    def stop(self):
-        raise NotImplementedError()
-
-    def process_tiles(self, tiles):
-        raise NotImplementedError()
-
-
-class TileWorkerPool(TileProcessor):
+class TileWorkerPool(object):
     """
     Manages multiple TileWorker.
     """
     def __init__(self, task, worker_class, size=2, dry_run=False, progress_logger=None):
-        TileProcessor.__init__(self, dry_run=dry_run)
         self.tiles_queue = queue_class(size)
         self.task = task
         self.dry_run = dry_run
@@ -128,9 +100,21 @@ class TileWorkerPool(TileProcessor):
                       For use when workers might be shutdown already by KeyboardInterrupt.
         """
         if not force:
+            alives = 0
             for proc in self.procs:
                 if proc.is_alive():
-                    self.tiles_queue.put(None)
+                    alives += 1
+
+            while alives:
+                # put None-sentinels to queue as long as we have workers alive
+                try:
+                    self.tiles_queue.put(None, timeout=1)
+                    alives -= 1
+                except Queue.Full:
+                    alives = 0
+                    for proc in self.procs:
+                        if proc.is_alive():
+                            alives += 1
 
         if force:
             timeout = 1.0
@@ -181,17 +165,14 @@ class TileCleanupWorker(TileWorker):
 class SeedProgress(object):
     def __init__(self, old_progress_identifier=None):
         self.progress = 0.0
-        self.eta = ETA()
         self.level_progress_percentages = [1.0]
-        self.level_progresses = []
+        self.level_progresses = None
+        self.level_progresses_level = 0
         self.progress_str_parts = []
-        self.old_level_progresses = None
-        if old_progress_identifier is not None:
-            self.old_level_progresses = old_progress_identifier
+        self.old_level_progresses = old_progress_identifier
 
     def step_forward(self, subtiles=1):
         self.progress += self.level_progress_percentages[-1] / subtiles
-        self.eta.update(self.progress)
 
     @property
     def progress_str(self):
@@ -199,53 +180,79 @@ class SeedProgress(object):
 
     @contextmanager
     def step_down(self, i, subtiles):
+        if self.level_progresses is None:
+            self.level_progresses = []
+        self.level_progresses = self.level_progresses[:self.level_progresses_level]
         self.level_progresses.append((i, subtiles))
+        self.level_progresses_level += 1
         self.progress_str_parts.append(status_symbol(i, subtiles))
         self.level_progress_percentages.append(self.level_progress_percentages[-1] / subtiles)
+
         yield
+
         self.level_progress_percentages.pop()
         self.progress_str_parts.pop()
-        self.level_progresses.pop()
+
+        self.level_progresses_level -= 1
+        if self.level_progresses_level == 0:
+            self.level_progresses = []
 
     def already_processed(self):
-        if self.old_level_progresses == []:
-            return True
-
-        if self.old_level_progresses is None:
-            return False
-
-        if self.progress_is_behind(self.old_level_progresses, self.level_progresses):
-            return True
-        else:
-            return False
+        return self.can_skip(self.old_level_progresses, self.level_progresses)
 
     def current_progress_identifier(self):
-        return self.level_progresses
+        if self.already_processed() or self.level_progresses is None:
+            return self.old_level_progresses
+        return self.level_progresses[:]
 
     @staticmethod
-    def progress_is_behind(old_progress, current_progress):
+    def can_skip(old_progress, current_progress):
         """
         Return True if the `current_progress` is behind the `old_progress` -
         when it isn't as far as the old progress.
 
-        >>> SeedProgress.progress_is_behind([], [(0, 1)])
-        True
-        >>> SeedProgress.progress_is_behind([(0, 1), (1, 4)], [(0, 1)])
+        >>> SeedProgress.can_skip(None, [(0, 4)])
         False
-        >>> SeedProgress.progress_is_behind([(0, 1), (1, 4)], [(0, 1), (0, 4)])
+        >>> SeedProgress.can_skip([], [(0, 4)])
         True
-        >>> SeedProgress.progress_is_behind([(0, 1), (1, 4)], [(0, 1), (1, 4)])
+        >>> SeedProgress.can_skip([(0, 4)], None)
+        False
+        >>> SeedProgress.can_skip([(0, 4)], [(0, 4)])
+        False
+        >>> SeedProgress.can_skip([(1, 4)], [(0, 4)])
         True
-        >>> SeedProgress.progress_is_behind([(0, 1), (1, 4)], [(0, 1), (3, 4)])
+        >>> SeedProgress.can_skip([(0, 4)], [(0, 4), (0, 4)])
         False
 
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (0, 4)])
+        False
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (0, 4), (1, 4)])
+        True
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (0, 4), (2, 4)])
+        False
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (0, 4), (3, 4)])
+        False
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (1, 4)])
+        False
+        >>> SeedProgress.can_skip([(0, 4), (0, 4), (2, 4)], [(0, 4), (1, 4), (0, 4)])
+        False
         """
-        for old, current in izip_longest(old_progress, current_progress, fillvalue=(9e15, 9e15)):
+        if current_progress is None:
+            return False
+        if old_progress is None:
+            return False
+        if old_progress == []:
+            return True
+        for old, current in izip_longest(old_progress, current_progress, fillvalue=None):
+            if old is None:
+                return False
+            if current is None:
+                return False
             if old < current:
                 return False
             if old > current:
                 return True
-        return True
+        return False
 
     def running(self):
         return True
@@ -258,6 +265,12 @@ class SeedInterrupted(Exception):
 
 
 class TileWalker(object):
+    """
+    TileWalker traverses through all tiles in a tile grid and calls worker_pool.process
+    for each (meta) tile. It traverses the tile grid (pyramid) depth-first.
+    Intersection with coverages are checked before handling subtiles in the next level,
+    allowing to determine if all subtiles should be seeded or skipped.
+    """
     def __init__(self, task, worker_pool, handle_stale=False, handle_uncached=False,
                  work_on_metatiles=True, skip_geoms_for_last_levels=0, progress_logger=None,
                  seed_progress=None):
@@ -271,13 +284,33 @@ class TileWalker(object):
         self.progress_logger = progress_logger
 
         num_seed_levels = len(task.levels)
-        self.report_till_level = task.levels[int(num_seed_levels * 0.8)]
+        if num_seed_levels >= 4:
+            self.report_till_level = task.levels[num_seed_levels-2]
+        else:
+            self.report_till_level = task.levels[num_seed_levels-1]
         meta_size = self.tile_mgr.meta_grid.meta_size if self.tile_mgr.meta_grid else (1, 1)
         self.tiles_per_metatile = meta_size[0] * meta_size[1]
         self.grid = MetaGrid(self.tile_mgr.grid, meta_size=meta_size, meta_buffer=0)
         self.count = 0
         self.seed_progress = seed_progress or SeedProgress()
 
+        # It is possible that we 'walk' through the same tile multiple times
+        # when seeding irregular tile grids[0]. limit_sub_bbox prevents that we
+        # recurse into the same area multiple times, but it is still possible
+        # that a tile is processed multiple times. Locking prevents that a tile
+        # is seeded multiple times, but it is possible that we count the same tile
+        # multiple times (in dry-mode, or while the tile is in the process queue).
+
+        # Tile counts can be off by 280% with sqrt2 grids.
+        # We keep a small cache of already processed tiles to skip most duplicates.
+        # A simple cache of 64 tile coordinates for each level already brings the
+        # difference down to ~8%, which is good enough and faster than a more
+        # sophisticated FIFO cache with O(1) lookup, or even caching all tiles.
+
+        # [0] irregular tile grids: where one tile does not have exactly 4 subtiles
+        # Typically when you use res_factor, or a custom res list.
+        self.seeded_tiles = {l: deque(maxlen=64) for l in task.levels}
+
     def walk(self):
         assert self.handle_stale or self.handle_uncached
         bbox = self.task.coverage.extent.bbox_for(self.tile_mgr.grid.srs)
@@ -318,7 +351,6 @@ class TileWalker(object):
         if current_level in levels:
             levels = levels[1:]
             process = True
-        current_level += 1
 
         for i, (subtile, sub_bbox, intersection) in enumerate(subtiles):
             if subtile is None: # no intersection
@@ -335,12 +367,19 @@ class TileWalker(object):
                     if self.seed_progress.already_processed():
                         self.seed_progress.step_forward()
                     else:
-                        self._walk(sub_bbox, levels, current_level=current_level,
+                        self._walk(sub_bbox, levels, current_level=current_level+1,
                             all_subtiles=all_subtiles)
 
             if not process:
                 continue
 
+            # check if subtile was already processed. see comment in __init__
+            if subtile in self.seeded_tiles[current_level]:
+                if not levels:
+                    self.seed_progress.step_forward(total_subtiles)
+                continue
+            self.seeded_tiles[current_level].appendleft(subtile)
+
             if not self.work_on_metatiles:
                 # collect actual tiles
                 handle_tiles = self.grid.tile_list(subtile)
@@ -403,7 +442,7 @@ class SeedTask(object):
 
     @property
     def id(self):
-        return self.md['name'], self.md['cache_name'], self.md['grid_name']
+        return self.md['name'], self.md['cache_name'], self.md['grid_name'], tuple(self.levels)
 
     def intersects(self, bbox):
         if self.coverage.contains(bbox, self.grid.srs): return CONTAINS
@@ -424,6 +463,10 @@ class CleanupTask(object):
         self.coverage = coverage
         self.complete_extent = complete_extent
 
+    @property
+    def id(self):
+        return 'cleanup', self.md['name'], self.md['cache_name'], self.md['grid_name']
+
     def intersects(self, bbox):
         if self.coverage.contains(bbox, self.grid.srs): return CONTAINS
         if self.coverage.intersects(bbox, self.grid.srs): return INTERSECTS
@@ -464,11 +507,18 @@ def seed_task(task, concurrency=2, dry_run=False, skip_geoms_for_last_levels=0,
     if task.refresh_timestamp is not None:
         task.tile_manager._expire_timestamp = task.refresh_timestamp
     task.tile_manager.minimize_meta_requests = False
+
+    work_on_metatiles = True
+    if task.tile_manager.rescale_tiles:
+        work_on_metatiles = False
+
     tile_worker_pool = TileWorkerPool(task, TileSeedWorker, dry_run=dry_run,
         size=concurrency, progress_logger=progress_logger)
     tile_walker = TileWalker(task, tile_worker_pool, handle_uncached=True,
         skip_geoms_for_last_levels=skip_geoms_for_last_levels, progress_logger=progress_logger,
-        seed_progress=seed_progress)
+        seed_progress=seed_progress,
+        work_on_metatiles=work_on_metatiles,
+    )
     try:
         tile_walker.walk()
     except KeyboardInterrupt:
