@@ -34,7 +34,8 @@ class WMSSource(MapLayer):
     supports_meta_tiles = True
     def __init__(self, client, image_opts=None, coverage=None, res_range=None,
                  transparent_color=None, transparent_color_tolerance=None,
-                 supported_srs=None, supported_formats=None, fwd_req_params=None):
+                 supported_srs=None, supported_formats=None, fwd_req_params=None,
+                 error_handler=None):
         MapLayer.__init__(self, image_opts=image_opts)
         self.client = client
         self.supported_srs = supported_srs or []
@@ -44,13 +45,38 @@ class WMSSource(MapLayer):
         self.transparent_color = transparent_color
         self.transparent_color_tolerance = transparent_color_tolerance
         if self.transparent_color:
-            self.transparent = True
+            self.image_opts.transparent = True
         self.coverage = coverage
         self.res_range = res_range
         if self.coverage:
             self.extent = MapExtent(self.coverage.bbox, self.coverage.srs)
         else:
             self.extent = DefaultMapExtent()
+        self.error_handler = error_handler
+
+    def is_opaque(self, query):
+        """
+        Returns true if we are sure that the image is not transparent.
+        """
+        if self.res_range and not self.res_range.contains(query.bbox, query.size,
+                                                          query.srs):
+            return False
+
+        if self.image_opts.transparent:
+            return False
+
+        if self.opacity is not None and (0.0 < self.opacity < 0.99):
+            return False
+
+        if not self.coverage:
+            # not transparent and no coverage
+            return True
+
+        if self.coverage.contains(query.bbox, query.srs):
+            # not transparent and completely inside coverage
+            return True
+
+        return False
 
     def get_map(self, query):
         if self.res_range and not self.res_range.contains(query.bbox, query.size,
@@ -67,6 +93,10 @@ class WMSSource(MapLayer):
             return resp
 
         except HTTPClientError as e:
+            if self.error_handler:
+                resp = self.error_handler.handle(e.response_code, query)
+                if resp:
+                    return resp
             log.warn('could not retrieve WMS map: %s', e)
             reraise_exception(SourceError(e.args[0]), sys.exc_info())
 
@@ -79,11 +109,6 @@ class WMSSource(MapLayer):
         if self.supported_srs:
             if query.srs not in self.supported_srs:
                 return self._get_transformed(query, format)
-            # some srs are equal but not the same (e.g. 900913/3857)
-            # use only supported srs so we use the right srs code.
-            idx = self.supported_srs.index(query.srs)
-            if self.supported_srs[idx] is not query.srs:
-                query.srs = self.supported_srs[idx]
         if self.extent and not self.extent.contains(MapExtent(query.bbox, query.srs)):
             return self._get_sub_query(query, format)
         resp = self.client.retrieve(query, format)
@@ -99,7 +124,7 @@ class WMSSource(MapLayer):
 
     def _get_transformed(self, query, format):
         dst_srs = query.srs
-        src_srs = self._best_supported_srs(dst_srs)
+        src_srs = self.supported_srs.best_srs(dst_srs)
         dst_bbox = query.bbox
         src_bbox = dst_srs.transform_bbox_to(src_srs, dst_bbox)
 
@@ -126,16 +151,6 @@ class WMSSource(MapLayer):
         img.format = format
         return img
 
-    def _best_supported_srs(self, srs):
-        latlong = srs.is_latlong
-
-        for srs in self.supported_srs:
-            if srs.is_latlong == latlong:
-                return srs
-
-        # else
-        return self.supported_srs[0]
-
     def _is_compatible(self, other, query):
         if not isinstance(other, WMSSource):
             return False
@@ -184,11 +199,14 @@ class WMSSource(MapLayer):
         )
 
 class WMSInfoSource(InfoSource):
-    def __init__(self, client, fi_transformer=None):
+    def __init__(self, client, fi_transformer=None, coverage=None):
         self.client = client
         self.fi_transformer = fi_transformer
+        self.coverage = coverage
 
     def get_info(self, query):
+        if self.coverage and not self.coverage.contains(query.coord, query.srs):
+            return None
         doc = self.client.get_info(query)
         if self.fi_transformer:
             doc = self.fi_transformer(doc)
