@@ -14,27 +14,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import division, with_statement
-import yaml
+from __future__ import division
+
 import time
-from mapproxy.srs import SRS
+
+import yaml
+import pytest
+
+from mapproxy.config.coverage import load_coverage
 from mapproxy.config.loader import (
     ProxyConfiguration,
     load_configuration,
     merge_dict,
     ConfigurationError,
 )
-from mapproxy.config.spec import validate_options
 from mapproxy.cache.tile import TileManager
+from mapproxy.config.spec import validate_options
+from mapproxy.seed.spec import validate_seed_conf
+from mapproxy.srs import SRS
 from mapproxy.test.helper import TempFile
 from mapproxy.test.unit.test_grid import assert_almost_equal_bbox
-from nose.tools import eq_, assert_raises
-from nose.plugins.skip import SkipTest
+from mapproxy.util.geom import EmptyGeometryError
+
 
 class TestLayerConfiguration(object):
     def _test_conf(self, yaml_part):
         base = {'sources': {'s': {'type': 'wms', 'req': {'url': ''}}}}
-        base.update(yaml.load(yaml_part))
+        base.update(yaml.safe_load(yaml_part))
         return base
 
     def test_legacy_ordered(self):
@@ -50,24 +56,25 @@ class TestLayerConfiguration(object):
                  title: Layer Three
                  sources: [s]
         ''')
-        conf = ProxyConfiguration(conf)
-        root = conf.wms_root_layer.wms_layer()
+        with pytest.warns(RuntimeWarning):
+            conf = ProxyConfiguration(conf)
+            root = conf.wms_root_layer.wms_layer()
 
         # no root layer defined
-        eq_(root.title, None)
-        eq_(root.name, None)
+        assert root.title == None
+        assert root.name == None
         layers = root.child_layers()
 
         # names are in order
-        eq_(layers.keys(), ['one', 'two', 'three'])
+        assert layers.keys() == ['one', 'two', 'three']
 
-        eq_(len(layers), 3)
-        eq_(layers['one'].title, 'Layer One')
-        eq_(layers['two'].title, 'Layer Two')
-        eq_(layers['three'].title, 'Layer Three')
+        assert len(layers) == 3
+        assert layers['one'].title == 'Layer One'
+        assert layers['two'].title == 'Layer Two'
+        assert layers['three'].title == 'Layer Three'
 
         layers_conf = conf.layers
-        eq_(len(layers_conf), 3)
+        assert len(layers_conf) == 3
 
     def test_legacy_unordered(self):
         conf = self._test_conf('''
@@ -82,21 +89,22 @@ class TestLayerConfiguration(object):
                 title: Layer Three
                 sources: [s]
         ''')
-        conf = ProxyConfiguration(conf)
-        root = conf.wms_root_layer.wms_layer()
+        with pytest.warns(RuntimeWarning):
+            conf = ProxyConfiguration(conf)
+            root = conf.wms_root_layer.wms_layer()
 
         # no root layer defined
-        eq_(root.title, None)
-        eq_(root.name, None)
+        assert root.title == None
+        assert root.name == None
         layers = root.child_layers()
 
         # names might not be in order
         # layers.keys() != ['one', 'two', 'three']
 
-        eq_(len(layers), 3)
-        eq_(layers['one'].title, 'Layer One')
-        eq_(layers['two'].title, 'Layer Two')
-        eq_(layers['three'].title, 'Layer Three')
+        assert len(layers) == 3
+        assert layers['one'].title == 'Layer One'
+        assert layers['two'].title == 'Layer Two'
+        assert layers['three'].title == 'Layer Three'
 
     def test_with_root(self):
         conf = self._test_conf('''
@@ -114,20 +122,20 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
 
-        eq_(root.title, 'Root Layer')
-        eq_(root.name, 'root')
+        assert root.title == 'Root Layer'
+        assert root.name == 'root'
         layers = root.child_layers()
 
         # names are in order
-        eq_(layers.keys(), ['root', 'one', 'two'])
+        assert layers.keys() == ['root', 'one', 'two']
 
-        eq_(len(layers), 3)
-        eq_(layers['root'].title, 'Root Layer')
-        eq_(layers['one'].title, 'Layer One')
-        eq_(layers['two'].title, 'Layer Two')
+        assert len(layers) == 3
+        assert layers['root'].title == 'Root Layer'
+        assert layers['one'].title == 'Layer One'
+        assert layers['two'].title == 'Layer Two'
 
         layers_conf = conf.layers
-        eq_(len(layers_conf), 2)
+        assert len(layers_conf) == 2
 
     def test_with_unnamed_root(self):
         conf = self._test_conf('''
@@ -144,12 +152,12 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
 
-        eq_(root.title, 'Root Layer')
-        eq_(root.name, None)
+        assert root.title == 'Root Layer'
+        assert root.name == None
 
         layers = root.child_layers()
         # names are in order
-        eq_(layers.keys(), ['one', 'two'])
+        assert layers.keys() == ['one', 'two']
 
     def test_without_root(self):
         conf = self._test_conf('''
@@ -164,12 +172,12 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
 
-        eq_(root.title, None)
-        eq_(root.name, None)
+        assert root.title == None
+        assert root.name == None
 
         layers = root.child_layers()
         # names are in order
-        eq_(layers.keys(), ['one', 'two'])
+        assert layers.keys() == ['one', 'two']
 
     def test_hierarchy(self):
         conf = self._test_conf('''
@@ -198,19 +206,19 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
 
-        eq_(root.title, 'Root Layer')
-        eq_(root.name, None)
+        assert root.title == 'Root Layer'
+        assert root.name == None
 
         layers = root.child_layers()
         # names are in order
-        eq_(layers.keys(), ['one', 'onea', 'oneb', 'oneba', 'onebb', 'two'])
+        assert layers.keys() == ['one', 'onea', 'oneb', 'oneba', 'onebb', 'two']
 
         layers_conf = conf.layers
-        eq_(len(layers_conf), 4)
-        eq_(layers_conf.keys(), ['onea', 'oneba', 'onebb', 'two'])
-        eq_(layers_conf['onea'].conf['title'], 'Layer One A')
-        eq_(layers_conf['onea'].conf['name'], 'onea')
-        eq_(layers_conf['onea'].conf['sources'], ['s'])
+        assert len(layers_conf) == 4
+        assert layers_conf.keys() == ['onea', 'oneba', 'onebb', 'two']
+        assert layers_conf['onea'].conf['title'] == 'Layer One A'
+        assert layers_conf['onea'].conf['name'] == 'onea'
+        assert layers_conf['onea'].conf['sources'] == ['s']
 
     def test_hierarchy_root_is_list(self):
         conf = self._test_conf('''
@@ -227,12 +235,12 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
 
-        eq_(root.title, 'Root Layer')
-        eq_(root.name, None)
+        assert root.title == 'Root Layer'
+        assert root.name == None
 
         layers = root.child_layers()
         # names are in order
-        eq_(layers.keys(), ['one', 'two'])
+        assert layers.keys() == ['one', 'two']
 
     def test_without_sources_or_layers(self):
         conf = self._test_conf('''
@@ -260,17 +268,17 @@ class TestGridConfiguration(object):
         conf = {}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['GLOBAL_MERCATOR'].tile_grid()
-        eq_(grid.srs, SRS(900913))
+        assert grid.srs == SRS(900913)
 
         grid = conf.grids['GLOBAL_GEODETIC'].tile_grid()
-        eq_(grid.srs, SRS(4326))
+        assert grid.srs == SRS(4326)
 
 
     def test_simple(self):
         conf = {'grids': {'grid': {'srs': 'EPSG:4326', 'bbox': [5, 50, 10, 55]}}}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        eq_(grid.srs, SRS(4326))
+        assert grid.srs == SRS(4326)
 
     def test_with_base(self):
         conf = {'grids': {
@@ -279,35 +287,35 @@ class TestGridConfiguration(object):
         }}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        eq_(grid.srs, SRS(4326))
+        assert grid.srs == SRS(4326)
 
     def test_with_num_levels(self):
         conf = {'grids': {'grid': {'srs': 'EPSG:4326', 'bbox': [5, 50, 10, 55], 'num_levels': 8}}}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        eq_(len(grid.resolutions), 8)
+        assert len(grid.resolutions) == 8
 
     def test_with_bbox_srs(self):
         conf = {'grids': {'grid': {'srs': 'EPSG:25832', 'bbox': [5, 50, 10, 55], 'bbox_srs': 'EPSG:4326'}}}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        assert_almost_equal_bbox([213372, 5538660, 571666, 6102110], grid.bbox, -3)
+        assert_almost_equal_bbox([213372, 5538660, 571666, 6102110], grid.bbox, 1)
 
     def test_with_min_res(self):
         conf = {'grids': {'grid': {'srs': 'EPSG:4326', 'bbox': [5, 50, 10, 55], 'min_res': 0.0390625}}}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        assert_almost_equal_bbox([5, 50, 10, 55], grid.bbox, 2)
-        eq_(grid.resolution(0), 0.0390625)
-        eq_(grid.resolution(1), 0.01953125)
+        assert_almost_equal_bbox([5, 50, 10, 55], grid.bbox)
+        assert grid.resolution(0) == 0.0390625
+        assert grid.resolution(1) == 0.01953125
 
     def test_with_max_res(self):
         conf = {'grids': {'grid': {'srs': 'EPSG:4326', 'bbox': [5, 50, 10, 55], 'max_res': 0.0048828125}}}
         conf = ProxyConfiguration(conf)
         grid = conf.grids['grid'].tile_grid()
-        assert_almost_equal_bbox([5, 50, 10, 55], grid.bbox, 2)
-        eq_(grid.resolution(0), 0.01953125)
-        eq_(grid.resolution(1), 0.01953125/2)
+        assert_almost_equal_bbox([5, 50, 10, 55], grid.bbox)
+        assert grid.resolution(0) == 0.01953125
+        assert grid.resolution(1) == 0.01953125/2
 
 class TestWMSSourceConfiguration(object):
     def test_simple_grid(self):
@@ -335,21 +343,21 @@ class TestWMSSourceConfiguration(object):
         conf = ProxyConfiguration(conf_dict)
 
         caches = conf.caches['osm'].caches()
-        eq_(len(caches), 1)
+        assert len(caches) == 1
         grid, extent, manager = caches[0]
 
-        eq_(grid.srs, SRS(4326))
-        eq_(grid.bbox, (5.0, 50.0, 10.0, 55.0))
+        assert grid.srs == SRS(4326)
+        assert grid.bbox == (5.0, 50.0, 10.0, 55.0)
 
         assert isinstance(manager, TileManager)
 
     def check_source_layers(self, conf_dict, layers):
         conf = ProxyConfiguration(conf_dict)
         caches = conf.caches['osm'].caches()
-        eq_(len(caches), 1)
+        assert len(caches) == 1
         grid, extent, manager = caches[0]
         source_layers = manager.sources[0].client.request_template.params.layers
-        eq_(source_layers, layers)
+        assert source_layers == layers
 
     def test_tagged_source(self):
         conf_dict = {
@@ -462,7 +470,7 @@ class TestWMSSourceConfiguration(object):
         conf = ProxyConfiguration(conf_dict)
         wms_layer = conf.layers['osm'].wms_layer()
         layers = wms_layer.map_layers[0].client.request_template.params.layers
-        eq_(layers, ['base', 'roads'])
+        assert layers == ['base', 'roads']
 
     def test_tagged_source_encoding(self):
         conf_dict = {
@@ -492,7 +500,7 @@ class TestWMSSourceConfiguration(object):
         conf = ProxyConfiguration(conf_dict)
         wms_layer = conf.layers['osm'].wms_layer()
         layers = wms_layer.map_layers[0].client.request_template.params.layers
-        eq_(layers, [u'☃'])
+        assert layers == [u'☃']
         # from cache
         self.check_source_layers(conf_dict, [u'☃'])
 
@@ -529,7 +537,7 @@ class TestBandMergeConfig(object):
             }
         }
         errors, informal_only = validate_options(conf_dict)
-        eq_(len(errors), 1)
+        assert len(errors) == 1
         assert "unknown 'f' in caches" in errors[0]
 
     def test_no_band_cache(self):
@@ -542,7 +550,7 @@ class TestBandMergeConfig(object):
             }
         }
         errors, informal_only = validate_options(conf_dict)
-        eq_(len(errors), 1)
+        assert len(errors) == 1
         assert "missing 'band', not in caches" in errors[0], errors
 
 
@@ -570,16 +578,18 @@ sources:
 """
 
     def test_loading(self):
-        with TempFile() as f:
-            open(f, 'wb').write(self.yaml_string)
-            services = load_services(f)
+        with TempFile() as tf:
+            with open(tf, 'wb') as f:
+                f.write(self.yaml_string)
+            services = load_services(tf)
         assert 'service' in services[0].names
 
     def test_loading_broken_yaml(self):
-        with TempFile() as f:
-            open(f, 'wb').write(b'\tbroken:foo')
+        with TempFile() as tf:
+            with open(tf, 'wb') as f:
+                f.write(b'\tbroken:foo')
             try:
-                load_services(f)
+                load_services(tf)
             except ConfigurationError:
                 pass
             else:
@@ -638,14 +648,14 @@ base: [%s]
                     config = load_configuration(cfg)
 
                     http = config.globals.get_value('http')
-                    eq_(http['client_timeout'], 1)
-                    eq_(http['headers']['bar'], 'qux')
-                    eq_(http['headers']['foo'], 'bar')
-                    eq_(http['headers']['baz'], 'quux')
-                    eq_(http['method'], 'GET')
+                    assert http['client_timeout'] == 1
+                    assert http['headers']['bar'] == 'qux'
+                    assert http['headers']['foo'] == 'bar'
+                    assert http['headers']['baz'] == 'quux'
+                    assert http['method'] == 'GET'
 
                     config_files = config.config_files()
-                    eq_(set(config_files.keys()), set([gp, p, cfg]))
+                    assert set(config_files.keys()) == set([gp, p, cfg])
                     assert abs(config_files[gp] - time.time()) < 10
                     assert abs(config_files[p] - time.time()) < 10
                     assert abs(config_files[cfg] - time.time()) < 10
@@ -656,25 +666,25 @@ class TestConfMerger(object):
         a = {'a': 1, 'b': [12, 13]}
         b = {}
         m = merge_dict(a, b)
-        eq_(a, m)
+        assert a == m
 
     def test_empty_conf(self):
         a = {}
         b = {'a': 1, 'b': [12, 13]}
         m = merge_dict(a, b)
-        eq_(b, m)
+        assert b == m
 
     def test_differ(self):
         a = {'a': 12}
         b = {'b': 42}
         m = merge_dict(a, b)
-        eq_({'a': 12, 'b': 42}, m)
+        assert {'a': 12 , 'b': 42} == m
 
     def test_recursive(self):
         a = {'a': {'aa': 12, 'a':{'aaa': 100}}}
         b = {'a': {'aa': 11, 'ab': 13, 'a':{'aaa': 101, 'aab': 101}}}
         m = merge_dict(a, b)
-        eq_({'a': {'aa': 12, 'ab': 13, 'a':{'aaa': 100, 'aab': 101}}}, m)
+        assert {'a': {'aa': 12, 'ab': 13, 'a':{'aaa': 100, 'aab': 101}}} == m
 
 
 class TestLoadConfiguration(object):
@@ -686,7 +696,8 @@ services:
                 """)
             load_configuration(f) # defaults to ignore_warnings=True
 
-            assert_raises(ConfigurationError, load_configuration, f, ignore_warnings=False)
+            with pytest.raises(ConfigurationError):
+                load_configuration(f, ignore_warnings=False)
 
 
 class TestImageOptions(object):
@@ -695,21 +706,21 @@ class TestImageOptions(object):
         }
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.globals.image_options.image_opts({}, 'image/png')
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, None)
-        eq_(image_opts.colors, 256)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bicubic')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == None
+        assert image_opts.colors == 256
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bicubic'
 
     def test_default_format_paletted_false(self):
         conf_dict = {'globals': {'image': { 'paletted': False }}}
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.globals.image_options.image_opts({}, 'image/png')
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, None)
-        eq_(image_opts.colors, None)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bicubic')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == None
+        assert image_opts.colors == None
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bicubic'
 
     def test_update_default_format(self):
         conf_dict = {'globals': {'image': {'formats': {
@@ -718,12 +729,12 @@ class TestImageOptions(object):
         }}}}
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.globals.image_options.image_opts({}, 'image/png')
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, None)
-        eq_(image_opts.colors, 16)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'nearest')
-        eq_(image_opts.encoding_options['quantizer'], 'mediancut')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == None
+        assert image_opts.colors == 16
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'nearest'
+        assert image_opts.encoding_options['quantizer'] == 'mediancut'
 
     def test_custom_format(self):
         conf_dict = {'globals': {'image': {'resampling_method': 'bilinear',
@@ -733,11 +744,11 @@ class TestImageOptions(object):
         }}}
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.globals.image_options.image_opts({}, 'image/foo')
-        eq_(image_opts.format, 'image/foo')
-        eq_(image_opts.mode, 'RGBA')
-        eq_(image_opts.colors, 42)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/foo'
+        assert image_opts.mode == 'RGBA'
+        assert image_opts.colors == 42
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
     def test_format_grid(self):
         conf_dict = {
@@ -756,11 +767,11 @@ class TestImageOptions(object):
         }
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.caches['test'].image_opts()
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, None)
-        eq_(image_opts.colors, 256)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == None
+        assert image_opts.colors == 256
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
     def test_custom_format_grid(self):
         conf_dict = {
@@ -794,18 +805,18 @@ class TestImageOptions(object):
         }
         conf = ProxyConfiguration(conf_dict)
         image_opts = conf.caches['test'].image_opts()
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, 'P')
-        eq_(image_opts.colors, 16)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == 'P'
+        assert image_opts.colors == 16
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
         image_opts = conf.caches['test2'].image_opts()
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, 'RGBA')
-        eq_(image_opts.colors, 8)
-        eq_(image_opts.transparent, True)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == 'RGBA'
+        assert image_opts.colors == 8
+        assert image_opts.transparent == True
+        assert image_opts.resampling == 'bilinear'
 
     def test_custom_format_source(self):
         conf_dict = {
@@ -841,36 +852,36 @@ class TestImageOptions(object):
         conf = ProxyConfiguration(conf_dict)
         _grid, _extent, tile_mgr = conf.caches['test'].caches()[0]
         image_opts = tile_mgr.image_opts
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, 'P')
-        eq_(image_opts.colors, 16)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == 'P'
+        assert image_opts.colors == 16
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
         image_opts = tile_mgr.sources[0].image_opts
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, 'P')
-        eq_(image_opts.colors, 256)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == 'P'
+        assert image_opts.colors == 256
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
 
         conf_dict['caches']['test']['request_format'] = 'image/tiff'
         conf = ProxyConfiguration(conf_dict)
         _grid, _extent, tile_mgr = conf.caches['test'].caches()[0]
         image_opts = tile_mgr.image_opts
-        eq_(image_opts.format, 'image/png')
-        eq_(image_opts.mode, 'P')
-        eq_(image_opts.colors, 16)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/png'
+        assert image_opts.mode == 'P'
+        assert image_opts.colors == 16
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
         image_opts = tile_mgr.sources[0].image_opts
-        eq_(image_opts.format, 'image/tiff')
-        eq_(image_opts.mode, None)
-        eq_(image_opts.colors, None)
-        eq_(image_opts.transparent, None)
-        eq_(image_opts.resampling, 'bilinear')
+        assert image_opts.format == 'image/tiff'
+        assert image_opts.mode == None
+        assert image_opts.colors == None
+        assert image_opts.transparent == None
+        assert image_opts.resampling == 'bilinear'
 
     def test_encoding_options_errors(self):
         conf_dict = {
@@ -923,3 +934,38 @@ class TestImageOptions(object):
 
         conf.globals.image_options.image_opts({}, 'image/jpeg')
 
+class TestCoverageValidation(object):
+    def test_union(self):
+        conf = {
+            'coverages': {
+                'covname': {
+                    'union': [
+                        {'bbox': [0, 0, 10, 10], 'srs': 'EPSG:4326'},
+                        {'bbox': [10, 0, 20, 10], 'srs': 'EPSG:4326', 'unknown': True},
+                    ],
+                },
+            },
+        }
+
+        errors, informal_only = validate_seed_conf(conf)
+        assert informal_only
+        assert len(errors) == 1
+        assert errors[0] == "unknown 'unknown' in coverages.covname.union[1]"
+
+
+class TestLoadCoverage(object):
+    def test_load_empty_geojson(self):
+        with TempFile() as tf:
+            with open(tf, 'wb') as f:
+                f.write(b'{"type": "FeatureCollection", "features": []}')
+            conf = {'datasource': tf, 'srs': 'EPSG:4326'}
+            with pytest.raises(EmptyGeometryError):
+                load_coverage(conf)
+
+    def test_load_empty_geojson_ogr(self):
+        with TempFile() as tf:
+            with open(tf, 'wb') as f:
+                f.write(b'{"type": "FeatureCollection", "features": []}')
+            conf = {'datasource': tf, 'where': '0 != 1', 'srs': 'EPSG:4326'}
+            with pytest.raises(EmptyGeometryError):
+                load_coverage(conf)
